from django import forms
from django.http import HttpResponseRedirect
from django.shortcuts import render, get_object_or_404

# Create your views here.
from django.urls import reverse_lazy, reverse
from django.views import generic
from django.views.generic import CreateView, UpdateView, RedirectView

from writingtogether.models import Story, StoryPart, StoryRound


class IndexView(generic.ListView):
    template_name = 'writingtogether/index.html'
    context_object_name = 'open_story_round_list'

    def get_queryset(self):
        # TODO: show open & finished rounds
        return StoryRound.objects.order_by('-created')[:5]


class DetailView(generic.DetailView):
    model = StoryRound
    template_name = 'writingtogether/detail.html'


class StoryRoundCreate(CreateView):
    model = StoryRound
    fields = ['name', 'participants', 'number_of_rounds']
    success_url = reverse_lazy('writing:index')

    def form_valid(self, form):
        self.object = form.save()
        sorted_participants = sorted(form.cleaned_data['participants'])
        number_of_participants = len(sorted_participants)

        for user in sorted_participants:
            story = Story.objects.create(
                part_of_round=self.object,
                started_by=user
            )

            previous_part=None
            for i in range(form.cleaned_data['number_of_rounds']):
                current_part = StoryPart.objects.create(
                    user=sorted_participants[i % number_of_participants],
                    previous_part=previous_part,
                    part_of=story
                )
                previous_part = current_part


class StoryUpdate(UpdateView):
    model = Story
    fields = ['name']


class StoryPartUpdate(UpdateView):
    model = StoryPart
    fields = ['text']
    template_name = 'writingtogether/story_part.html'

    #def form_valid(self, form):
    #    form.instance.created_by = self.request.user
    #    form.instance.previous_part_id = self.kwargs['previous']
    #    form.instance.part_of_id = self.kwargs['story_pk']
    #    return super().form_valid(form)


class RedirectToNextOpenPart(RedirectView):
    permanent = False
    query_string = True
    pattern_name = 'writing:update_story_part'

    def get_redirect_url(self, *args, **kwargs):
        story_round = get_object_or_404(StoryRound, pk=kwargs['story_round_pk'])

        story_part = story_round.get_next_story_part(user=self.request.user)

        kwargs['story_pk'] = story_part.part_of.pk
        kwargs['story_part_pk'] = story_part.pk
        return super().get_redirect_url(*args, **kwargs)
